@@ -404,10 +404,6 @@ var messageHeaderSink = {
         }
       }
 
-      // display name optimization. Eliminate any large quantities of white space from the display name.
-      // such that Hello       World.txt becomes Hello World.txt.
-      displayName = displayName.replace(/ +/g, " ");
-          
       currentAttachments.push (new createNewAttachmentInfo(contentType, url, displayName, uri, isExternalAttachment));
       // if we have an attachment, set the MSG_FLAG_ATTACH flag on the hdr
       // this will cause the "message with attachment" icon to show up
@@ -1074,19 +1070,27 @@ function detachAttachment(aAttachment, aSaveFirst)
                            aAttachment.messageUri, aSaveFirst);
 }
 
+/**
+ * Return true if possible attachments in the currently loaded message can be
+ * deleted/detached.
+ */
 function CanDetachAttachments()
 {
   var uri = GetLoadedMessage();
   var canDetach = !IsNewsMessage(uri) && (!IsImapMessage(uri) || MailOfflineMgr.isOnline());
   if (canDetach && ("content-type" in currentHeaderData))
-  {
-    var contentType = currentHeaderData["content-type"].headerValue;
-    canDetach = contentType.indexOf("application/x-pkcs7-mime") < 0 && 
-        contentType.indexOf("application/x-pkcs7-signature") < 0;
-  }
+    canDetach = !ContentTypeIsSMIME(currentHeaderData["content-type"].headerValue);
   return canDetach;
 }
 
+/** Return true if the content type is an S/MIME one. */
+function ContentTypeIsSMIME(contentType)
+{
+// S/MIME is application/pkcs7-mime and application/pkcs7-signature
+// - also match application/x-pkcs7-mime and application/x-pkcs7-signature.
+  return /application\/(x-)?pkcs7-(mime|signature)/.test(contentType);
+}
+
 function onShowAttachmentContextMenu()
 {
   // if no attachments are selected, disable the Open and Save...
@@ -1094,49 +1098,58 @@ function onShowAttachmentContextMenu()
   var selectedAttachments = attachmentList.selectedItems;
   var openMenu = document.getElementById('context-openAttachment');
   var saveMenu = document.getElementById('context-saveAttachment');
+  var saveAllMenu = document.getElementById('context-saveAllAttachments');
   var detachMenu = document.getElementById('context-detachAttachment');
   var deleteMenu = document.getElementById('context-deleteAttachment');
   var detachAllMenu = document.getElementById('context-detachAllAttachments');
   var deleteAllMenu = document.getElementById('context-deleteAllAttachments');
+
   var canDetach = CanDetachAttachments();
-  var canOpen = false;
-  for (var i = 0; i < selectedAttachments.length && !canOpen; i++)
-    canOpen = selectedAttachments[i].attachment.contentType != 'text/x-moz-deleted';
-  if (canOpen && selectedAttachments.length == 1)
+  var deletedAmongSelected = false;
+  var detachedAmongSelected = false;
+  var anyDeleted = false; // at least one deleted attachment in the list
+  var anyDetached = false; // at least one detached attachment in the list
+  var selectNone = selectedAttachments.length == 0;
+
+  // Check if one or more of the selected attachments are deleted.
+  for (var i = 0; i < selectedAttachments.length && !deletedAmongSelected; i++)
+    deletedAmongSelected =
+      (selectedAttachments[i].attachment.contentType == 'text/x-moz-deleted');
+
+  // Check if one or more of the selected attachments are detached.
+  for (var i = 0; i < selectedAttachments.length && !detachedAmongSelected; i++)
+    detachedAmongSelected = selectedAttachments[i].attachment.isExternalAttachment;
+
+  // Check if any attachments are deleted.
+  for (var i = 0; i < currentAttachments.length && !anyDeleted; i++)
+    anyDeleted = (currentAttachments[i].contentType == 'text/x-moz-deleted');
+
+  // Check if any attachments are detached.
+  for (var i = 0; i < currentAttachments.length && !anyDetached; i++)
+    anyDetached = currentAttachments[i].isExternalAttachment;
+
+  openMenu.setAttribute('disabled', selectNone);
+  saveMenu.setAttribute('disabled', selectNone);
+  detachMenu.setAttribute('disabled', selectNone);
+  deleteMenu.setAttribute('disabled', selectNone);
+  saveAllMenu.setAttribute('disabled', !selectNone);
+  detachAllMenu.setAttribute('disabled', !selectNone);
+  deleteAllMenu.setAttribute('disabled', !selectNone);
+
+  if (!selectNone)
   {
-    openMenu.removeAttribute('disabled');
+    openMenu.setAttribute('disabled', deletedAmongSelected);
+    saveMenu.setAttribute('disabled', deletedAmongSelected);
+    detachMenu.setAttribute('disabled', !canDetach || deletedAmongSelected
+                                        || detachedAmongSelected);
+    deleteMenu.setAttribute('disabled', !canDetach || deletedAmongSelected
+                                        || detachedAmongSelected);
   }
   else
   {
-    openMenu.setAttribute('disabled', true);
-  }
-  if (canOpen)
-  {
-    saveMenu.removeAttribute('disabled');
-  }
-  else
-  {
-    saveMenu.setAttribute('disabled', true);
-  }
-  if (canDetach && canOpen)
-  {
-    detachMenu.removeAttribute('disabled');
-    deleteMenu.removeAttribute('disabled');
-  }
-  else
-  {
-    detachMenu.setAttribute('disabled', 'true');
-    deleteMenu.setAttribute('disabled', 'true');
-  }
-  if (canDetach)
-  {
-    detachAllMenu.removeAttribute('disabled');
-    deleteAllMenu.removeAttribute('disabled');
-  }
-  else
-  {
-    detachAllMenu.setAttribute('disabled', 'true');
-    deleteAllMenu.setAttribute('disabled', 'true');
+    saveAllMenu.setAttribute('disabled', anyDeleted);
+    detachAllMenu.setAttribute('disabled', !canDetach || anyDeleted || anyDetached);
+    deleteAllMenu.setAttribute('disabled', !canDetach || anyDeleted || anyDetached);
   }
 }
 
@@ -1191,6 +1204,16 @@ function cloneAttachment(aAttachment)
   return obj;
 }
 
+function createAttachmentDisplayName(aAttachment)
+{
+  // Strip any white space at the end of the display name to avoid
+  // attachment name spoofing (especially Windows will drop trailing dots
+  // and whitespace from filename extensions). Leading and internal
+  // whitespace will be taken care of by the crop="center" attribute.
+  // We must not change the actual filename, though.
+  return aAttachment.displayName.replace(/\s+$/, "");
+}
+
 function displayAttachmentsForExpandedView()
 {
   var numAttachments = currentAttachments.length;
@@ -1221,9 +1244,10 @@ function displayAttachmentsForExpandedView()
     {
       var attachment = currentAttachments[index];
 
-      // create a new attachment widget. set the label, set the 
-      // moz-icon img src
-      var attachmentView = attachmentList.appendItem(attachment.displayName);
+      // Create a new attachment widget
+      var displayName = createAttachmentDisplayName(attachment);
+      var attachmentView = attachmentList.appendItem(displayName);
+      
       attachmentView.setAttribute("class", "descriptionitem-iconic"); 
 
       if (gShowLargeAttachmentView)
@@ -1290,26 +1314,25 @@ function FillAttachmentListPopup(popup)
   // First clear out the old view...
   ClearAttachmentMenu(popup);
 
+  var canDetachOrDeleteAll = CanDetachAttachments();
+
   for (index in currentAttachments)
   {
     ++attachmentIndex;
     addAttachmentToPopup(popup, currentAttachments[index], attachmentIndex);
+    if (canDetachOrDeleteAll &&
+        (currentAttachments[index].isExternalAttachment ||
+        currentAttachments[index].contentType == 'text/x-moz-deleted'))
+      canDetachOrDeleteAll = false;
   }
 
   gBuildAttachmentPopupForCurrentMsg = false;
+
   var detachAllMenu = document.getElementById('file-detachAllAttachments');
   var deleteAllMenu = document.getElementById('file-deleteAllAttachments');
-  if (CanDetachAttachments())
-  {
-    detachAllMenu.removeAttribute('disabled');
-    deleteAllMenu.removeAttribute('disabled');
-  }
-  else
-  {
-    detachAllMenu.setAttribute('disabled', 'true');
-    deleteAllMenu.setAttribute('disabled', 'true');
-  }
 
+  detachAllMenu.setAttribute('disabled', !canDetachOrDeleteAll);
+  deleteAllMenu.setAttribute('disabled', !canDetachOrDeleteAll);
 }
 
 // Public method used to clear the file attachment menu
@@ -1349,11 +1372,13 @@ function addAttachmentToPopup(popup, attachment, attachmentIndex)
       while (popup.childNodes[indexOfSeparator].localName != 'menuseparator')
         indexOfSeparator++;      
 
+      var displayName = createAttachmentDisplayName(attachment);
       var formattedDisplayNameString = gMessengerBundle.getFormattedString("attachmentDisplayNameFormat",
-                                       [attachmentIndex, attachment.displayName]);
+                                       [attachmentIndex, displayName]);
 
-      item.setAttribute('label', formattedDisplayNameString); 
-      item.setAttribute('accesskey', attachmentIndex); 
+      item.setAttribute("crop", "center");
+      item.setAttribute('label', formattedDisplayNameString);
+      item.setAttribute('accesskey', attachmentIndex);
 
       // each attachment in the list gets its own menupopup with options for saving, deleting, detaching, etc. 
       var openpopup = document.createElement('menupopup'); 
@@ -1385,20 +1410,7 @@ function addAttachmentToPopup(popup, attachment, attachmentIndex)
       if (!gDeleteLabelAccesskey)
         gDeleteLabelAccesskey = gMessengerBundle.getString("deleteLabelAccesskey");
 
-      // we should also check if an attachment has been detached...
-      // but that uses X-Mozilla-External-Attachment-URL, which
-      // we'd need to check for somehow.
-
-      var signedOrEncrypted = false;
-      if ("content-type" in currentHeaderData)
-      {
-        var contentType = currentHeaderData["content-type"].headerValue;
-        signedOrEncrypted = contentType.indexOf("application/x-pkcs7-mime") >= 0 || 
-            contentType.indexOf("application/x-pkcs7-signature") >= 0;
-      }
-      var canDetach = !(/news-message:/.test(attachment.uri)) && 
-          !signedOrEncrypted &&
-          (!(/imap-message/.test(attachment.uri)) || MailOfflineMgr.isOnline());
+      var canDetach = CanDetachAttachments() && !attachment.isExternalAttachment;
       menuitementry.setAttribute('label', gOpenLabel); 
       menuitementry.setAttribute('accesskey', gOpenLabelAccesskey); 
       menuitementry = openpopup.appendChild(menuitementry);
